from typing import Any, List, Union
import ldap3, pytest
from pytest_console_scripts import ScriptRunner, RunResult
from pytest_mock import MockerFixture
from pyphonebook import PhoneBook, PhoneBookEntry


@pytest.fixture
def mock_ldap(mocker: MockerFixture) -> None:
    print('Mocking LDAP server')
    def fake_server(self, server):
        fake_connection = ldap3.Connection(ldap3.Server('fake_server'), client_strategy=ldap3.MOCK_SYNC)
        fake_connection.strategy.entries_from_json('pyphonebook/tests/fake_data.json')
        fake_connection.bind()

        return fake_connection

    mocker.patch('pyphonebook.PhoneBook._getLDAP', new=fake_server)


@pytest.fixture
def phonebook(mock_ldap: pytest.fixture) -> PhoneBook:
    pb = PhoneBook()
    yield pb


@pytest.fixture
def mock_users(phonebook: pytest.fixture) -> Union[List[PhoneBookEntry], List[Any]]:
    yield phonebook.search("(cn=*)", raw=True)

# Convenience function to make it simpler to specify command lines
def command(self: ScriptRunner, cmd: str = '') -> RunResult:
    cmd = 'pyphonebook {}'.format(cmd).split()
    return self.run(*cmd)
ScriptRunner.command = command

# Reduce the boilerplate in checking script output
def check(self: ScriptRunner, command: str = '', stdout: List[str] = [], stderr: List[str] = [], returncode: int = 0) -> RunResult:
    ret = self.command(command)
    assert ret.returncode == returncode

    if stderr:
        assert ret.stderr == '\n'.join(stderr)+'\n'
    else:
        assert ret.stderr == ''

    if stdout:
        assert ret.stdout == '\n'.join(stdout)+'\n'
    else:
        assert ret.stdout == ''

    return ret
ScriptRunner.check = check

@pytest.fixture
def pyphonebook(mock_ldap: pytest.fixture, script_runner: ScriptRunner) -> ScriptRunner:
    yield script_runner
